﻿using System.Runtime.InteropServices;
using System.Text;

internal class Program
{
    [DllImport("kernel32.dll")]
    static extern long GetCurrentThreadId();

    [DllImport("kernel32.dll", SetLastError = true)]
    public static extern int VirtualQuery(UIntPtr lpAddress, ref MEMORY_BASIC_INFORMATION lpBuffer, int dwLength);

    [DllImport("kernel32.dll")]
    static extern void GetCurrentThreadStackLimits(out ulong LowLimit, out ulong HightLimit);

    static unsafe void Main(string[] args)
    {
        Console.WriteLine($"ThreadID: {GetCurrentThreadId()}");

        GetCurrentThreadStackLimits(out ulong stackLowLimit, out ulong stackHightLimit);
        Console.WriteLine($"Stack: 0x{stackLowLimit:X16} ~ 0x{stackHightLimit:X16}, Size: {(stackHightLimit - stackLowLimit) / 1024} KB");

        for (ulong page = stackLowLimit; page < stackHightLimit;)
        {
            MEMORY_BASIC_INFORMATION mbi = new MEMORY_BASIC_INFORMATION();
            int result = VirtualQuery((nuint)page, ref mbi, Marshal.SizeOf(mbi));
            if (result != 0)
            {
                WritePageInfo((nuint)page, mbi);
            }

            page += (nuint)mbi.RegionSize;
        }

        Console.ReadLine();
    }

    private static void WritePageInfo(nuint page, MEMORY_BASIC_INFORMATION mbi)
    {
        string text = $"BaseAddress: 0x{mbi.BaseAddress:X16}, Size: {mbi.RegionSize / 1024} KB, {GetPageState(mbi.State)} {GetPageAccess(mbi.Protect)}";
        Console.WriteLine($"{text}");
    }

    private static string GetPageAccess(uint state)
    {
        StringBuilder sb = new StringBuilder();

        if ((state & 0x10) == 0x10)
        {
            sb.Append("PAGE_EXECUTE, ");
        }

        if ((state & 0x01) == 0x01)
        {
            sb.Append("PAGE_NOACCESS, ");
        }

        if ((state & 0x04) == 0x04)
        {
            sb.Append("PAGE_READWRITE, ");
        }

        if ((state & 0x100) == 0x100)
        {
            sb.Append("PAGE_GUARD");
        }

        return sb.ToString();
    }

    private static string GetPageState(uint state)
    {
        StringBuilder sb = new StringBuilder();

        if ((state & 0x1000) == 0x1000)
        {
            sb.Append("MEM_COMMIT, ");
        }

        if ((state & 0x10000) == 0x10000)
        {
            sb.Append("MEM_FREE, ");
        }

        if ((state & 0x2000) == 0x2000)
        {
            sb.Append("MEM_RESERVE, ");
        }

        return sb.ToString();
    }


    private unsafe static nuint GetThreadStackBasse()
    {
        MEMORY_BASIC_INFORMATION mbi = new MEMORY_BASIC_INFORMATION();

        nuint stackPtr = new nuint((void*)(&mbi));
        int result = VirtualQuery(stackPtr, ref mbi, Marshal.SizeOf(mbi));
        if (result != 0)
        {
            return mbi.AllocationBase;
        }

        return 0;
    }
}

[StructLayout(LayoutKind.Sequential)]
public struct MEMORY_BASIC_INFORMATION
{
    public UIntPtr BaseAddress;
    public UIntPtr AllocationBase;
    public uint AllocationProtect;
    public IntPtr RegionSize;
    public uint State;
    public uint Protect;
    public uint Type;
}